## Revisione: aggiunto regime di rotazione diverso per ogni marcia

## Funzione per il calcolo dell'accelerazione e frenata in un tratto rettilineo
##
## DATI:
##
## m = massa totale veicolo+pilota [kg]
## p = passo del veicolo [m]. Puo' essere un vettore contenente piu' valori del passo
## b = distanza longitudinale baricentro dall'asse posteriore [m].
##     Puo' essere un vettore contenente diverse percentuali
## h = altezza baricentro [m]. Puo' essere un vettore contenente diverse altezze
## Rr = raggio ruota posteriore [m]
## Rf = raggio ruota anteriore [m]
## I_r = Inerzia ruota posteriore [kg m^2]
## I_f = Inerzia ruota anteriore [kg m^2]
## I_m = Inerzia motore ridotta all'albero [kg m^2]
## I_p = Inerzia cambio albero primario [kg m^2]
## I_s = Inerzia cambio albero secondario [kg m^2]
## Rc = vettore rapporti del cambio [w_motore/w_ruota]
## R_p = Rapporto primario [w_albero_motore/w_primario_cambio]
## R_f = Rapporto finale [w_secondario_cambio/w_ruota]
## CC = matrice coppia motore. La prima riga contiene il regime di rotazione del
##      motore [rpm], la seconda riga il valore di coppia a quel regime [Nm]
## rs = regime di cambio marcia, deve essere un vettore con tanti elementi
##      quanti sono i rapporti del cambio [rpm]
## Af = area frontale [m^2]
## Cd = coefficiente di resistenza aerodinamica
## Cl = coefficiente di portanza aerodinamica
## Cm = coefficiente di momento aerodinamico. Negativo = picchiante.
##      Si considera già moltiplicato per la corda del profilo.
## rho = densita' dell'aria [kg/m^3]
## mu = coefficiente di attrito gomma/asfalto
## mur = coefficiente resistenza rotolamento
## alpha = inclinazione strada [rad]
## Lr = lunghezza del tratto rettilineo [m]. Puo' essere un vettore, contenente
##      piu' di una lunghezza
## vi = velocita' iniziale [m/s]. Puo' essere un vettore contenente piu' velocita'
## vf = velocita' finale [m/s]. Puo' essere un vettore contenente piu' velocita'
## ns = numero di suddivisioni del tratto rettilineo
## fdu = freno da usare:
##       "f" = solo anteriore
##       "fr" = anteriore + posteriore
## tr = trazione:
##      "f" = trazione anteriore
##      "r" = trazione posteriore
##
## La funzione restituisce 9 vettori:
## tt = vettore tempo [s]
## xx = vettore spazio [m]
## vv = vettore velocita' [m/s]
## rpmm = vettore regime di rotazione del motore [rad/s]
## ccm = coppia del motore [Nm]
## ccr = coppia alla ruota [Nm]
## pgas = percentuale apertura gas [%]
## Nr = carico ruota posteriore [N]
## Nf = carico ruota anteriore [N]
##
## Questi vettori possono essere multidimensionali, e conterranno:
##
## prima dimensione: lunghezze Lr
## seconda dimensione: velocita' iniziale vi
## terza dimensione: velocita' finale vf
## quarta dimensione: distanza longitudinale baricentro b
## quinta dimensione: altezza baricentro h
## sesta dimensione: passo p
## settima dimensione: variazione dx

function [tt, xx, vv, rpmm, ccm, ccr, pgas, Nr, Nf] = rett(m, p, b, h, Rr, Rf, I_r, I_f, I_m, I_p, I_s, Rc, R_p, R_f, CC, rs, Af, Cd, Cl, Cm, rho, mu, mur, alpha, Lr, vi, vf, ns, fdu, tr)
  g = 9.81;       ## accelerazione di gravita'
  
  ## Lavoro con un vettore 1D contenente tutte le combinazioni possibili delle
  ## variabili, disposto in modo tale che utilizzando la funzione reshape sia
  ## poi possibile ricostruire la matrice N-dimensionale
  ## Lunghezza vettore 1D
  l1D = length(Lr)*length(vi)*length(vf)*length(b)*length(h)*length(p);
  
  ## Creo i vettori 1D da usare per il calcolo
  if size(p)(1) > 1 && size(p)(2) == 1
    p = p';
  endif
  p_v = p.*ones(l1D/length(p),length(p));
  p_v = reshape(p_v, [l1D, 1]);
  
  if size(h)(1) > 1 && size(h)(2) == 1
    h = h';
  endif
  h_v = h.*ones(l1D/length(p)/length(h),length(h));
  h_v = repmat(h_v, 1, length(p));
  h_v = reshape(h_v, [l1D, 1]);
  
  if size(b)(1) > 1 && size(b)(2) == 1
    b = b';
  endif
  b_v = b.*ones(l1D/length(p)/length(h)/length(b),length(b));
  b_v = repmat(b_v, 1, length(p)*length(h));
  b_v = reshape(b_v, [l1D, 1]);
  
  if size(vf)(1) > 1 && size(vf)(2) == 1
    vf = vf';
  endif
  vf_v = vf.*ones(l1D/length(p)/length(h)/length(b)/length(vf),length(vf));
  vf_v = repmat(vf_v, 1, length(p)*length(h)*length(b));
  vf_v = reshape(vf_v, [l1D, 1]);
  
  if size(vi)(1) > 1 && size(vi)(2) == 1
    vi = vi';
  endif
  vi_v = vi.*ones(l1D/length(p)/length(h)/length(b)/length(vf)/length(vi),length(vi));
  vi_v = repmat(vi_v, 1, length(p)*length(h)*length(b)*length(vf));
  vi_v = reshape(vi_v, [l1D, 1]);
  
  if size(Lr)(1) > 1 && size(Lr)(2) == 1
    Lr = Lr';
  endif
  Lr_v = Lr.*ones(l1D/length(p)/length(h)/length(b)/length(vf)/length(vi)/length(Lr),length(Lr));
  Lr_v = repmat(Lr_v, 1, length(p)*length(h)*length(b)*length(vf)*length(vi));
  Lr_v = reshape(Lr_v, [l1D, 1]);
  
  if size(rs)(1) > 1 && size(rs)(2) == 1
    rs = rs';
  endif
  
  vrr = 2*pi/60*CC(1,:);    ## Vettore regimi di rotazione, S.I.
  vcr = CC(2,:);            ## Vettore coppie, S.I.
  rs = 2*pi/60*rs;          ## Regime cambio marcia, S.I.
  Vrr = ones(l1D, 1).*vrr;
  Vcr = ones(l1D, 1).*vcr;
  
  ## Voglio che Rc sia un vettore riga
  if size(Rc)(1) > 1 && size(Rc)(2) == 1
    Rc = Rc';
  endif
  vrc = ones(l1D,1).*Rc;
  
  ## Calcolo il regime di rotazione massimo della ruota motrice e la velocita'
  ## massima del veicolo in ogni marcia
  Rmax = rs./Rc;
  Vrmax = Rr*Rmax;
  
  ## Tratto di spazio finito
  dx = Lr_v/ns;
  
  ## Creo i vettori velocita' in accelerazione e frenata. Inizialmente, sono
  ## riempiti col valore della velocita' iniziale e finale rispettivamente.
  ## I vettori risultanti sono vettori colonna.
  vva = repmat(vi_v, ns, 1);
  vvf = repmat(vf_v, ns, 1);
  
  ## Altri dati utili
  rpmm = zeros(l1D*ns, 1);             ## Regime di rotazione del motore
  ccm = zeros(l1D*ns, 1);              ## Coppia del motore
  ccr = zeros(l1D*ns, 1);              ## Coppia alla ruota
  pgas = zeros(l1D*ns, 1);             ## Percentuale apertura gas
  Nr = zeros(l1D*ns, 1);               ## Carico ruota posteriore
  Nf = zeros(l1D*ns, 1);               ## Carico ruota anteriore
  for ii = 1:ns
    ## Velocita' attuale in tutte le configurazioni
    vvac = vva(l1D*(ii-1)+1:ii*l1D);
    vvfc = vvf(l1D*(ii-1)+1:ii*l1D);
    
    ## Calcolo le forze aerodinamiche alla velocita' attuale
    Fda = 0.5*rho*Af*Cd*vvac.^2;     ## Resistenza accelerazione
    Lda = 0.5*rho*Af*Cl*vvac.^2;     ## Portanza accelerazione
    Mda = 0.5*rho*Af*Cm.*vvac.^2;    ## Momento accelerazione
    Fdf = 0.5*rho*Af*Cd*vvfc.^2;     ## Resistenza frenata
    Ldf = 0.5*rho*Af*Cl*vvfc.^2;     ## Portanza frenata
    Mdf = 0.5*rho*Af*Cm.*vvfc.^2;    ## Momento frenata
    
    ## Regime e rapporto attuale
    rpmr = vvac/Rr;                                      ## Regime di rotazione ruota motrice. Vettore colonna
    vrpm = rpmr.*Rc.*R_p.*R_f;                           ## Regime di rotazione motore. Colonne: rapporti da 1 a max
    irpm = vrpm<=rs;                                     ## Matrice per eliminare i regimi superiori a quello max
    rpmm(l1D*(ii-1)+1:ii*l1D) = max(irpm.*vrpm, [], 2);  ## Regime di rotazione motore
    rdc = max(irpm.*vrc, [], 2);                         ## Rapporto del cambio
    
    ## Coppia motore attuale
    srr = Vrr>=rpmm(l1D*(ii-1)+1:ii*l1D);
    tsrr = [zeros(rows(srr),1), srr(:,1:end-1)];
    srr = srr-tsrr;
    srl = [srr(:,2:end), zeros(rows(srr),1)];
    rsucc = sum(srr.*Vrr, 2);                       ## Regime motore successivo
    rprec = sum(srl.*Vrr, 2);                       ## Regime motore precedente
    csucc = sum(srr.*Vcr, 2);                       ## Coppia motore successivo
    cprec = sum(srl.*Vcr, 2);                       ## Coppia motore precedente
    delY = csucc-cprec;
    delX = rsucc-rprec;
    slope = (delX>0).*delY./delX+(delX<=0).*rprec;
    inter = csucc-slope.*rsucc;
    ccm(l1D*(ii-1)+1:ii*l1D) = slope.*rpmm(l1D*(ii-1)+1:ii*l1D)+inter;  ## Coppia motore
    ccr(l1D*(ii-1)+1:ii*l1D) = ccm(l1D*(ii-1)+1:ii*l1D).*rdc.*R_f.*R_p; ## Coppia alla ruota
    
    ## Accelerazione e frenata. Per quanto riguarda la frenata, la considero
    ## come se fosse un accelerazione e giro il vettore alla fine.
    tau_r = 1/Rr;         ## Rapporto di velocità rotazione ruota posteriore / avanzamento
    tau_f = 1/Rf;         ## Rapporto di velocità rotazione ruota anteriore / avanzamento
    if tr == "r"
      tau_s = R_f/Rr;           ## Rapporto di velocità rotazione secondario cambio / avanzamento
      tau_p = rdc*R_f/Rr;       ## Rapporto di velocità rotazione primario cambio / avanzamento
      tau_m = R_p*rdc*R_f/Rr;   ## Rapporto di velocità rotazione albero motore / avanzamento
      
      ## Accelerazione, sollevamento ruota anteriore:
      ## Massa equivalente da aggiungere alla massa totale
      meq_ai = I_r.*tau_r.^2+I_s.*tau_s.^2+I_p.*tau_p.^2+I_m.*tau_m.^2;
      ## Carico ruota posteriore
      Nr_ai = m*g*cos(alpha)-Lda;
      ## Forza di trazione
      Ft = (Nr_ai.*(b_v+mur*h_v)-Mda)./h_v;
      ## Accelerazione
      ai = (Ft-Nr_ai*mur-Fda-m*g*sin(alpha))./(m+meq_ai);
      
      ## Accelerazione, slittamento ruota posteriore:
      ## Massa equivalente da aggiungere alla massa totale
      meq_as = I_r.*tau_r.^2+I_f.*tau_f.^2+I_s.*tau_s.^2+I_p.*tau_p.^2+I_m.*tau_m.^2;
      ## Carico ruota anteriore
      Nf_as = ((Lda-m*g*cos(alpha)).*(h_v.*(mu-mur)-b_v)-Mda)./(p_v-mu*h_v);
      ## Carico ruota posteriore
      Nr_as = m*g*cos(alpha)-Lda-Nf_as;
      ## Accelerazione
      as = (Nr_as*(mu-mur)-Nf_as*mur-Fda-m*g*sin(alpha))./(m+meq_as);
      
      ## Accelerazione, potenza del motore
      ## Forza di spinta del motore
      Frm = ccr(l1D*(ii-1)+1:ii*l1D)./Rr;
      ## Carico sulla ruota anteriore
      Nf_m = ((b_v+mur.*h_v).*(m*g*cos(alpha)-Lda)-Frm.*h_v-Mda)./p_v;
      ## Carico sulla ruota posteriore
      Nr_m = m*g*cos(alpha)-Nf_m-Lda;
      ## Accelerazione
      am = (Frm-(Nr_m+Nf_m).*mur-m*g*sin(alpha)-Fda)./(m+meq_as);
      
      ## Accelerazione risultante
      aa = min([ai,as,am], [], 2);
      
      ## Massa equivalente per calcolo percentuale apertura gas
      if aa == ai
        meq = meq_ai;
      elseif aa == as
        meq = meq_as;
      else
        meq = meq_as;
      endif
      
      ## Carichi sulle ruote
      Nr(l1D*(ii-1)+1:ii*l1D) = (aa==ai).*Nr_ai+(aa==as).*Nr_as+(aa==am).*Nr_m;
      Nf(l1D*(ii-1)+1:ii*l1D) = (aa==ai).*0+(aa==as).*Nf_as+(aa==am).*Nf_m;
    elseif tr == "f"
      tau_s = R_f/Rf;           ## Rapporto di velocità rotazione secondario cambio / avanzamento
      tau_p = rdc*R_f/Rf;       ## Rapporto di velocità rotazione primario cambio / avanzamento
      tau_m = R_p*rdc*R_f/Rf;   ## Rapporto di velocità rotazione albero motore / avanzamento
      
      ## Accelerazione, slittamento ruota anteriore
      ## Massa equivalente da aggiungere alla massa totale
      meq_as = I_r.*tau_r.^2+I_f.*tau_f.^2+I_s.*tau_s.^2+I_p.*tau_p.^2+I_m.*tau_m.^2;
      ## Carico ruota anteriore
      Nf_as = ((b_v+mur*h_v).*(m*g*cos(alpha)-Lda)-Mda)./(p_v+mu*h_v);
      ## Carico ruota posteriore
      Nr_as = m*g*cos(alpha)-Lda-Nf_as;
      ## Accelerazione
      as = (Nf_as.*(mu-mur)-Nr_as*mur-Fda-m*g*sin(alpha))./(m+meq_as);
      
      ## Accelerazione, potenza del motore
      ## Forza di spinta del motore
      Frm = ccr(l1D*(ii-1)+1:ii*l1D)./Rf;
      ## Carico sulla ruota anteriore
      Nf_m = ((b_v+mur.*h_v).*(m*g*cos(alpha)-Lda)-Frm.*h_v-Mda)./p_v;
      ## Carico sulla ruota posteriore
      Nr_m = m*g*cos(alpha)-Nf_m-Lda;
      ## Accelerazione
      am = (Frm-mur*(Nf_m+Nr_m)-m*g*sin(alpha)-Fda)./(m+meq_as);
      
      ## Accelerazione risultante
      aa = min([as,am], [], 2);
      
      ## Massa equivalente per calcolo percentuale apertura gas
      meq = meq_as;
      
      ## Carichi sulle ruote
      Nr(l1D*(ii-1)+1:ii*l1D) = (aa==as).*Nr_as+(aa==am).*Nr_m;
      Nf(l1D*(ii-1)+1:ii*l1D) = (aa==as).*Nf_as+(aa==am).*Nf_m;
    endif
    
    ## Frenata
    if fdu == "f"
      ## Frenata, sollevamento ruota posteriore
      ## Massa equivalente da aggiungere alla massa totale
      meq_air = I_f.*tau_f.^2+I_r.*tau_r.^2;
      ## Carico ruota anteriore
      Nf_air = m*g*cos(alpha)-Lda;
      Ff_air = (Nf_air.*(p_v-b_v-mur*h_v)+Mda)./h_v;
      ## Decelerazione
      air = (Nf_air*mur+Ff_air+m*g*sin(alpha)+Fda)./(m+meq_air);
      
      ## Frenata, slittamento ruota anteriore
      ## Massa equivalente da aggiungere alla massa totale
      meq_asf = I_f.*tau_f.^2+I_r.*tau_r.^2;
      ## Carico ruota anteriore
      Nf_asf = ((b_v+mur*h_v).*(m*g*cos(alpha)-Ldf)-Mdf)./(p_v-mu*h_v);
      ## Carico ruota posteriore
      Nr_asf = m*g*cos(alpha)-Ldf-Nf_asf;
      ## Decelerazione
      asf = (Nr_asf*mur+Nf_asf*(mu+mur)+m*g*sin(alpha)+Fdf)./(m+meq_asf);
      
      ## Decelerazione risultante
      af = min([air,asf], [], 2);
      
      ## Carichi sulle ruote
      Nr(l1D*(ii-1)+1:ii*l1D) = (af==asf).*Nr_asf;
      Nf(l1D*(ii-1)+1:ii*l1D) = (af==air).*Nf_air+(af==asf).*Nf_asf;
    elseif fdu == "fr"
      ## Frenata, sollevamento ruota posteriore
      ## Massa equivalente da aggiungere alla massa totale
      meq_air = I_f.*tau_f.^2+I_r.*tau_r.^2;
      ## Carico ruota anteriore
      Nf_air = m*g*cos(alpha)-Lda;
      Ff_air = (Nf_air.*(p_v-b_v-mur*h_v)+Mda)./h_v;
      ## Decelerazione
      air = (Nf_air*mur+Ff_air+m*g*sin(alpha)+Fda)./(m+meq_air);
      
      ## Frenata, slittamento ruote
      ## Massa equivalente da aggiungere alla massa totale
      meq_asf = I_f.*tau_f.^2+I_r.*tau_r.^2;
      ## Carico ruota anteriore
      Nf_asf = ((m*g*cos(alpha)-Lda)./(b_v+h_v*(mu+mur))-Mda)./p_v;
      ## Carico ruota posteriore
      Nr_asf = m*g*cos(alpha)-Nf_asf-Lda;
      ## Decelerazione
      asf = ((Nf_asf+Nr_asf).*(mu+mur)+m*g*sin(alpha)+Fda)./(m+meq_asf);
      
      ## Decelerazione risultante
      af = min([air,asf], [], 2);
      
      ## Carichi sulle ruote
      Nr(l1D*(ii-1)+1:ii*l1D) = (af==asf).*Nr_asf;
      Nf(l1D*(ii-1)+1:ii*l1D) = (af==air).*Nf_air+(af==asf).*Nf_asf;
    endif
    
    ## Calcolo l'apertura percentuale del gas
    if tr == "r"
      ## La percentuale deve essere zero se ccr = 0
      ccr_mod = ccr(l1D*(ii-1)+1:ii*l1D);
      ccr_mod(ccr_mod<=0) = 10e6;
      pgas(l1D*(ii-1)+1:ii*l1D) = (aa.*(m+meq)+Fda).*Rr./ccr_mod*100;
    elseif tr == "f"
      ccr_mod = ccr(l1D*(ii-1)+1:ii*l1D);
      ccr_mod(ccr_mod<=0) = 10e6;
      pgas(l1D*(ii-1)+1:ii*l1D) = (aa.*(m+meq)+Fda).*Rf./ccr_mod*100;
    endif
    
    ## Calcolo la velocita' successiva in accelerazione e frenata
    vva(ii*l1D+1:l1D*(ii+1)) = sqrt(2*aa.*dx+vvac.^2);
    vvf(ii*l1D+1:l1D*(ii+1)) = sqrt(2*af.*dx+vvfc.^2);
    
    ## Visualizzazione stato simulazione
    ## disp(["Percentuale completamento: ", num2str(ii/ns*100), " %"]);
  endfor
  rpmm = [rpmm; rpmm(end-l1D+1:end)];
  ccm = [ccm; ccm(end-l1D+1:end)];
  ccr = [ccr; ccr(end-l1D+1:end)];
  pgas = [pgas; pgas(end-l1D+1:end)];
  
  ## Giro il vettore della frenata
  vvf = reshape(vvf, [l1D, length(vvf)/l1D]);
  vvf = flip(vvf, 2);
  vvf = reshape(vvf, [l1D*(ns+1), 1]);
  
  ## Metto insieme i vettori accelerazione e frenata
  temp = vva<vvf;
  vvf = (vva>=vvf).*vvf;
  vva = temp.*vva;
  rpmm = temp.*rpmm;
  ccm = temp.*ccm;
  ccr = temp.*ccr;
  pgas = temp.*pgas;
  vv = vvf+vva;
  
  ## Creo vettori spazio e tempo
  xx = linspace(0, Lr_v, ns+1);     ## Vettore spazio totale
  xx = reshape(xx, [l1D*(ns+1), 1]);
  dx = repmat(dx, ns+1, 1);         ## Vettore delta spazio
  dt = dx./vv;                      ## Vettore delta tempo
  tt = zeros(length(xx),1);         ## Vettore tempo totale
  for ii = 1:ns
    tt(l1D*ii+1:l1D*(ii+1)) = tt(l1D*(ii-1)+1:l1D*ii)+dt(l1D*(ii-1)+1:l1D*ii);
  endfor
  
  ## Ricombino i vettori 1D per trasformarli in 7D
  vv = reshape(vv, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
  tt = reshape(tt, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
  xx = reshape(xx, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
  rpmm = reshape(rpmm, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
  ccm = reshape(ccm, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
  ccr = reshape(ccr, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
  pgas = reshape(pgas, [length(Lr), length(vi), length(vf), length(b), length(h), length(p), ns+1]);
endfunction
